package ec.edu.epn.seguridad.VO;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.naming.InitialContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.jasig.cas.client.util.AbstractCasFilter;
import org.jasig.cas.client.validation.Assertion;
import org.primefaces.model.DefaultTreeNode;
import org.primefaces.model.TreeNode;
import org.primefaces.model.menu.DefaultMenuItem;
import org.primefaces.model.menu.DefaultMenuModel;
import org.primefaces.model.menu.DefaultSeparator;
import org.primefaces.model.menu.DefaultSubMenu;
import org.primefaces.model.menu.MenuModel;

import ec.edu.epn.seguridad.servicio.ServicioSeguridad;
import ec.edu.epn.seguridad.vo.Menu;
import ec.edu.epn.seguridad.vo.Perfil;
import ec.edu.epn.seguridad.vo.Usuario;

/**
 * Maneja la pantalla de seleccin de rol, la informacin del menu dinmico e
 * incluye funciones para validar accesos a nivel de componentes web, ejemplo:
 * si un botn no se debe presentar si el usuario no tiene permiso de creacin
 * se colocara el atributo rendered="${sesionUsuario.puedeCrear}"
 * 
 * 
 */
public class SesionUsuario implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 2171946028044573953L;

	public static String NOMBRE_VARIABLE_ACCESOS = "EPN_ACCESOS";

	public int UNIDAD_USUARIO_LOGEADO = 0;
	public String nombre_usuario_logeado = " ";
	public long id_usuario_log = 0;
	public String id_sessionU = "";

	private ServicioSeguridad ss;
	{
		try {
			InitialContext ctx = new InitialContext();
			ss = (ServicioSeguridad) ctx.lookup(
					"java:global/ServiciosSeguridadEPN/ServicioSeguridadBean!ec.edu.epn.seguridad.servicio.ServicioSeguridad");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private String perfilSeleccionado;

	private Map<String, String> opciones;

	private TreeNode root;
	private TreeNode selectedNode;

	private MenuModel model;	
	

	// Contiene los datos del usuario logeado
	private Usuario usuario;

	/**
	 * Retorna todos los perfiles que tiene el usuario logeado
	 * 
	 * @return
	 */
	public List<SelectItem> getPerfiles() {
		List<SelectItem> perfiles = new ArrayList<SelectItem>();
		String nombreUsuario = getUsuario().getNombreUsuario();
		List<Perfil> resultadoBusqueda = ss.obtenerPerfilesUsuario(nombreUsuario);
		if (resultadoBusqueda != null) {
			for (Perfil p : resultadoBusqueda) {
				SelectItem i = new SelectItem();
				i.setDescription(p.getNombre());
				i.setValue(p.getNombre());
				i.setLabel(p.getNombre());
				perfiles.add(i);
			}
		}
		return perfiles;
	}

	/**
	 * Se ejecuta al momento de hacer click en el boton Aceptar de la pantalla
	 * Seleccionar Rol
	 */
	public String seleccionRol(String idAplicativo) {
		String nombreUsuario = getUsuario().getNombreUsuario();
		long varIdAplicativo= Long.valueOf(idAplicativo);
		if (perfilSeleccionado != null && ss.usuarioTienePerfil(nombreUsuario, perfilSeleccionado)) {
			llenarMenus(perfilSeleccionado, varIdAplicativo);
			cargarRol(perfilSeleccionado);
			return "principal";
		} else {
			return null;
		}
	}

	public TreeNode getRoot() {
		return root;
	}

	public TreeNode getSelectedNode() {
		return selectedNode;
	}

	public void setSelectedNode(TreeNode selectedNode) {
		this.selectedNode = selectedNode;
	}

	

	public void llenarMenus(String peril, long idAplicativo) {		

		model = new DefaultMenuModel();		
		
		FacesContext fc = FacesContext.getCurrentInstance();
		String urlAplicacion = fc.getExternalContext().getInitParameter("urlAplicacion");
		
		List<Menu> menusPadre = ss.obtenerMenusPadreAplicacion(peril.trim(), idAplicativo);

		DefaultSeparator separator = new DefaultSeparator();
		separator.setStyleClass("fa fa-ellipsis-v menu-separator");

		for (Menu m : menusPadre) {
			DefaultSubMenu firstSubmenu = new DefaultSubMenu(m.getNombre().trim().concat("   "));
			firstSubmenu.setStyle("font-family: 'calibri Light'; font-size: 11px;");			
			firstSubmenu.setIcon("fa fa-caret-right");			
			firstSubmenu.setExpanded(true);
			 

			List<Menu> menusHijo = ss.obtenerMenusHijo(peril.trim(), m.getId(), "menu", "opcion");

			for (Menu mi : menusHijo) {

				if (mi.getTipo().trim().equals("menu")) {
					DefaultSubMenu secondSubmenu = new DefaultSubMenu(mi.getNombre().trim());
					secondSubmenu.setStyle("font-family: 'calibri Light'; font-size: 11px;");
					secondSubmenu.setIcon("fa fa-folder");

					List<Menu> opcionesMenu = ss.obtenerMenusHijo(peril.trim(), mi.getId(), "opcion", "");

					for (Menu mo : opcionesMenu) {
						DefaultMenuItem item = new DefaultMenuItem(mo.getNombre().trim());
						item.setStyle("font-family: 'calibri Light'; font-size: 11px;");
						item.setUrl(urlAplicacion + mo.getUrl());						
						secondSubmenu.addElement(item);

					}

					firstSubmenu.addElement(secondSubmenu);

				} else {

					DefaultMenuItem item = new DefaultMenuItem(mi.getNombre().trim());
					item.setStyle("font-family: 'calibri Light'; font-size: 11px;");
					item.setUrl(urlAplicacion + mi.getUrl());					
					firstSubmenu.addElement(item);

				}

			}

			model.addElement(firstSubmenu);
			model.addElement(separator);			
		}
	}

	private void cargarRol(String rol) {
		Map<String, String> opciones = new HashMap<String, String>();
		FacesContext fc = FacesContext.getCurrentInstance();
		String urlAplicacion = fc.getExternalContext().getInitParameter("urlAplicacion");

		root = new DefaultTreeNode(new MenuAccion(null), null);

		List<Menu> menus = ss.obtenerMenusPorRol(rol);
		List<Menu> menusPadre = ss.obtenerMenusPadre(rol);

		TreeNode[] listPadre = new TreeNode[menusPadre.size()];

		for (int i = 0; i < menusPadre.size(); i++) {

			listPadre[i] = new DefaultTreeNode(new MenuAccion(menusPadre.get(i)), root);

			List<Menu> menusHijo = ss.obtenerMenusHijo(rol, menusPadre.get(i).getId(), "menu", "opcion");
			TreeNode[] listHijo = new TreeNode[menusHijo.size()];

			for (int j = 0; j < menusHijo.size(); j++) {

				listHijo[j] = new DefaultTreeNode(new MenuAccion(menusHijo.get(j)), listPadre[i]);

				List<Menu> opcionesMenu = ss.obtenerMenusHijo(rol, menusHijo.get(j).getId(), "opcion", "");
				TreeNode[] listOpciones = new TreeNode[opcionesMenu.size()];

				for (int k = 0; k < opcionesMenu.size(); k++) {
					listOpciones[k] = new DefaultTreeNode(new MenuAccion(opcionesMenu.get(k)), listHijo[j]);
				}
			}
		}

		for (Menu m : menus) {
			if (m.getUrlCompleto() != null && m.getTipo() != "menu" && m.getUrlCompleto().startsWith(urlAplicacion)) {
				opciones.put(m.getUrl(), m.getPermisos());
			}
		}
		setOpciones(opciones);
	}

	

	/**
	 * Permite que aparezcan como expandidas las carpetas de segundo nivel
	 */

	/**
	 * Obtiene el usuario de la variable de session en donde publica el Filtro
	 * de Autorizacion de CAS y lo guarda en un atributo para que a partir de la
	 * segunda vez se consulte directamente de este atributo
	 */
	public Usuario getUsuario() {
		if (usuario == null) {
			FacesContext fc = FacesContext.getCurrentInstance();
			HttpServletRequest request = (HttpServletRequest) fc.getExternalContext().getRequest();
			HttpSession session = request.getSession();
			Assertion a = (Assertion) session.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION);
			String nombreUsuario = a.getPrincipal().getName();
			usuario = ss.obtenerUsuario(nombreUsuario);

			id_sessionU = (String) session.getId();

			session.setAttribute("unidadUsuarioLogueado", usuario.getId_unidad());
			UNIDAD_USUARIO_LOGEADO = (Integer) session.getAttribute("unidadUsuarioLogueado");

			session.setAttribute("NombreUsuarioLog", usuario.getNombreUsuario());
			nombre_usuario_logeado = (String) session.getAttribute("NombreUsuarioLog");

			session.setAttribute("IdUsuarioLog", usuario.getId());
			id_usuario_log = (Long) session.getAttribute("IdUsuarioLog");

		}
		return usuario;
	}

	/**
	 * Retorna el campo permisos de la tabla autorizaciones para la url actual
	 */
	public String getAccesos() {
		FacesContext fc = FacesContext.getCurrentInstance();
		HttpServletRequest request = (HttpServletRequest) fc.getExternalContext().getRequest();
		return (String) request.getAttribute(NOMBRE_VARIABLE_ACCESOS);
	}

	/**
	 * Valida que el usuario tenga permisos en la tabla autorizaciones para la
	 * url actual
	 */
	public String getPermisosAcceso(String permisoAcceso) {
		FacesContext fc = FacesContext.getCurrentInstance();
		HttpServletRequest request = (HttpServletRequest) fc.getExternalContext().getRequest();
		String acceso = (String) request.getAttribute(NOMBRE_VARIABLE_ACCESOS);

		char delimiter = ' ';
		String accesoToken = " ";
		accesoToken = acceso;

		String puedeLeer = "";
		String puedeCrear = "";
		String puedeModificar = "";
		String puedeEliminar = "";

		if (acceso != null) {
			int N = 0;
			for (int i = 0; i < accesoToken.length(); i++)
				if (accesoToken.charAt(i) == delimiter)
					N++;
			String[] tokens = new String[N + 1];

			int right = 0, left = 0;
			for (int i = 0; i < N; i++) {
				while (accesoToken.charAt(right) != delimiter)
					right++;
				tokens[i] = accesoToken.substring(left, right);
				right++;
				left = right;
			}
			tokens[N] = accesoToken.substring(right, accesoToken.length());

			for (int i = 0; i < tokens.length; i++) {
				if (i == 0 && permisoAcceso == "l") {
					puedeLeer = tokens[i];

					if (puedeLeer.equals("l")) {
						return "false";
					} else if (puedeLeer.equals("-") || puedeLeer.equals(" ")) {
						return "true";
					}
				} else if (i == 1 && permisoAcceso == "c") {
					puedeCrear = tokens[i];
					if (puedeCrear.equals("c")) {
						return "false";
					} else if (puedeCrear.equals("-") || puedeCrear.equals(" ")) {
						return "true";
					}
				} else if (i == 2 && permisoAcceso == "m") {
					puedeModificar = tokens[i];
					if (puedeModificar.equals("m")) {
						return "false";
					} else if (puedeModificar.equals("-") || puedeModificar.equals(" ")) {
						return "true";
					}
				} else if (i == 3 && permisoAcceso == "e") {
					puedeEliminar = tokens[i];
					if (puedeEliminar.equals("e")) {
						return "false";
					} else if (puedeEliminar.equals("-")) {
						return "true";
					}
				}
			}
		}
		return "true";
	}

	/**
	 * Permiso de acceso puede crear
	 */
	public String getAccesoCrear() {
		return getPermisosAcceso("c");
	}

	/**
	 * Permiso de acceso puede modificar
	 */
	public String getAccesoModificar() {
		return getPermisosAcceso("m");
	}

	/**
	 * Permiso de acceso puede eliminar
	 */
	public String getAccesoEliminar() {
		return getPermisosAcceso("e");
	}

	public String getPerfilSeleccionado() {
		return perfilSeleccionado;
	}

	public void setPerfilSeleccionado(String perfilSeleccionado) {
		this.perfilSeleccionado = perfilSeleccionado;
	}

	public Map<String, String> getOpciones() {
		return opciones;
	}

	public void setOpciones(Map<String, String> opciones) {
		this.opciones = opciones;
	}

	public MenuModel getModel() {
		return model;
	}

	public void setModel(MenuModel model) {
		this.model = model;
	}

}
